@@ -13,11 +13,7 @@ class ReLU(Function, Module):
         return xp.maximum(0, x)
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         x = self.parent_tensors[0]
-        if x.requires_grad:
-            go = self._cast_like(grad_output, x.data)
-            x_grad = go * (x.data > 0)
-        else:
-            x_grad = None
+        x_grad = grad_output * (x.data > 0) if x.requires_grad else None
         return x_grad
 
 
@@ -34,8 +30,7 @@ class ReLU6(Function, Module):
         x = self.parent_tensors[0]
         if x.requires_grad:
             mask = (x.data > 0) & (x.data < 6)
-            go = self._cast_like(grad_output, x.data)
-            x_grad = go * mask
+            x_grad = grad_output * mask
         else:
             x_grad = None
         return x_grad
@@ -51,11 +46,7 @@ class Sigmoid(Function, Module):
         return self.sigmoid_x
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         x = self.parent_tensors[0]
-        if x.requires_grad:
-            go = self._cast_like(grad_output, x.data)
-            x_grad = go * self.sigmoid_x * (1 - self.sigmoid_x)
-        else:
-            x_grad = None
+        x_grad = grad_output * self.sigmoid_x * (1 - self.sigmoid_x) if x.requires_grad else None
         return x_grad
 
 
@@ -74,11 +65,8 @@ class Softmax(Function, Module):
         return self.softmax_x
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         x = self.parent_tensors[0]
-        # Cast for compute consistency
-        go = self._cast_like(grad_output, x.data)
-        sm = self.softmax_x.astype(x.data.dtype, copy=False)
-        dot_product = xp.sum(sm * go, axis=self.axis, keepdims=True)
-        x_grad = (sm * (go - dot_product)) if x.requires_grad else None
+        dot_product = xp.sum(self.softmax_x * grad_output, axis=self.axis, keepdims=True)
+        x_grad = self.softmax_x * (grad_output - dot_product) if x.requires_grad else None
         return x_grad
 
 class Tanh(Function, Module):
@@ -92,11 +80,7 @@ class Tanh(Function, Module):
         return self.tanh_x
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         x = self.parent_tensors[0]
-        if x.requires_grad:
-            go = self._cast_like(grad_output, x.data)
-            x_grad = go * (1 - self.tanh_x ** 2)
-        else:
-            x_grad = None
+        x_grad = grad_output * (1 - self.tanh_x ** 2) if x.requires_grad else None
         return x_grad
 
 class LeakyReLU(Function, Module):
@@ -109,11 +93,7 @@ class LeakyReLU(Function, Module):
         return xp.where(x >= 0, x, self.negative_slope * x)
     def backward(self, grad_output: xp.ndarray) -> xp.ndarray:
         x = self.parent_tensors[0]
-        if x.requires_grad:
-            go = self._cast_like(grad_output, x.data)
-            x_grad = go * xp.where(x.data >= 0, 1, self.negative_slope)
-        else:
-            x_grad = None
+        x_grad = grad_output * xp.where(x.data >= 0, 1, self.negative_slope) if x.requires_grad else None
         return x_grad
     
 
@@ -143,4 +123,4 @@ def softmax(x , axis: int = -1):
 def tanh(x):
         return Tanh()(x)
 def leaky_relu(x, negative_slope: float = 0.01):
-        return LeakyReLU(negative_slope=negative_slope)(x)
+        return LeakyReLU(negative_slope=negative_slope)(x)
\ No newline at end of file
