@@ -19,24 +19,21 @@ class Function(ABC):
                     processed_inputs.append(Tensor(data, requires_grad=False))
                 except Exception as e:
                     raise TypeError(f"Input {i} must be convertible to numpy array, got {type(inp)}") from e
-
-        # Prepare compute-time arrays following autocast policy (no Cast nodes)
-        op_name = getattr(self, 'name', None) or self.__class__.__name__
-        compute_inputs = None
+        
+        # Apply autocast if enabled (but not for Cast operations to avoid recursion)
         try:
             from neurograd.amp.autocast import is_autocast_enabled
             if is_autocast_enabled():
-                from neurograd.amp.utils import maybe_cast_data
-                compute_inputs = [maybe_cast_data(inp, op_name=op_name) for inp in processed_inputs]
+                from neurograd.amp.utils import maybe_cast_tensor
+                op_name = getattr(self, 'name', None) or self.__class__.__name__
+                if op_name != 'Cast':  # Avoid recursion with Cast operations
+                    processed_inputs = [maybe_cast_tensor(inp, op_name=op_name) for inp in processed_inputs]
         except ImportError:
-            # AMP not available; fall back to raw data
-            compute_inputs = None
-
-        if compute_inputs is None:
-            compute_inputs = [inp.data for inp in processed_inputs]
-
+            # AMP not available, continue with original precision
+            pass
+        
         self.parent_tensors = processed_inputs
-        output_data = self.forward(*compute_inputs)
+        output_data = self.forward(*[inp.data for inp in processed_inputs])
         requires_grad = any(inp.requires_grad for inp in processed_inputs)
         output = Tensor(output_data, requires_grad=requires_grad, grad_fn=self)
         return output
@@ -83,17 +80,3 @@ class Function(ABC):
                 grad = xp.sum(grad, axis=i, keepdims=True)
         
         return grad
-
-    # ---- AMP helpers (centralized) ----
-    def _cast_like(self, arr, ref: xp.ndarray):
-        """Cast array/tensor `arr` to dtype of `ref` without copying when possible."""
-        if arr is None:
-            return None
-        data = arr.data if isinstance(arr, Tensor) else arr
-        try:
-            return data if data.dtype == ref.dtype else data.astype(ref.dtype, copy=False)
-        except Exception:
-            return data
-
-    # Note: Each op's backward should return grads matching the dtypes of
-    # their corresponding inputs, mirroring PyTorch's autograd contract.
