@@ -64,7 +64,7 @@ class RMSprop(Optimizer):
             "beta": self.beta,
             "eps": self.eps,
             "weight_decay": self.weight_decay,
-            "params": self.params,
+            "t": self.t,
             "momentum": self.momentum,
         }
 
@@ -73,8 +73,8 @@ class RMSprop(Optimizer):
         self.beta = state_dict["beta"]
         self.eps = state_dict["eps"]
         self.weight_decay = state_dict["weight_decay"]
-        self.params = state_dict["params"]
-        self.momentum = state_dict["momentum"]
+        self.t = state_dict["t"]
+        self.momentum = [(n, xp.array(a)) for n, a in state_dict["momentum"]]
     
     def __repr__(self) -> str:
         return f"RMSprop(lr={self.lr}, beta={self.beta}, eps={self.eps}, weight_decay={self.weight_decay})."
